use std::env;

use clap;
use log::{error, info, LevelFilter};
use svtl::{get_base_url, get_content, get_relative_links, YtDlpWrapper};

fn init_logger() {
    let mut builder = env_logger::Builder::from_env("RUST_LOG");
    if env::var("RUST_LOG").is_err() {
        builder.filter_level(LevelFilter::Info);
    }
    builder.init();
}

#[tokio::main]
async fn main() {
    init_logger();

    let matches = clap::Command::new("Svt Links")
        .version("1.0")
        .about("Download entire series from SVT")
        .arg(
            clap::Arg::new("link")
                .short('l')
                .long("link")
                .required(true)
                .help("Link to series"),
        )
        .get_matches();

    let yt_dlp = match YtDlpWrapper::new().await {
        Some(yt_dlp) => yt_dlp,
        None => {
            error!("yt-dlp not found in PATH. Please install yt-dlp to proceed.");
            return;
        }
    };

    let master_link = matches.get_one::<String>("link").unwrap().as_ref();
    let base_url = match get_base_url(master_link) {
        Ok(url) => url,
        Err(e) => {
            error!("Failed to parse url with error: {:?}", &e);
            return;
        }
    };

    let content = match get_content(master_link).await {
        Ok(content) => content,
        Err(e) => {
            error!("Failed to get base page: {}", e);
            return;
        }
    };

    let links = get_relative_links(&content);
    info!("Found the following links: {}", links.join(", "));

    for link_part in links {
        let link = format!("{}{}", base_url, link_part);
        info!("Downloading: {}", link);
        if let Err(e) = yt_dlp.spawn_yt_dlp_task(&link).await {
            error!("Failed to download with error: {:?}", &e);
            return;
        }
    }
}
